@@ -1,3 +1,8 @@
+import sys
+if sys.version_info < (3, 7):
+    raise ImportError('Your Python version {0} is not supported by aiogram, please install '
+                      'Python 3.7+'.format('.'.join(map(str, sys.version_info[:3]))))
+
 import asyncio
 import os
 
@@ -20,7 +25,7 @@ else:
     if 'DISABLE_UVLOOP' not in os.environ:
         asyncio.set_event_loop_policy(uvloop.EventLoopPolicy())
 
-__all__ = [
+__all__ = (
     'Bot',
     'Dispatcher',
     '__api_version__',
@@ -35,8 +40,8 @@ __all__ = [
     'md',
     'middlewares',
     'types',
-    'utils'
-]
+    'utils',
+)
 
-__version__ = '2.2'
-__api_version__ = '4.3'
+__version__ = '2.11.1'
+__api_version__ = '5.0'
