@@ -3,17 +3,20 @@ import contextlib
 import io
 import ssl
 import typing
+import warnings
 from contextvars import ContextVar
-from typing import Dict, List, Optional, Union
+from typing import Dict, List, Optional, Union, Type
 
 import aiohttp
 import certifi
 from aiohttp.helpers import sentinel
 
 from . import api
+from .api import TelegramAPIServer, TELEGRAM_PRODUCTION
 from ..types import ParseMode, base
 from ..utils import json
-from ..utils.auth_widget import check_token
+from ..utils.auth_widget import check_integrity
+from ..utils.deprecated import deprecated
 
 
 class BaseBot:
@@ -32,7 +35,8 @@ class BaseBot:
             proxy_auth: Optional[aiohttp.BasicAuth] = None,
             validate_token: Optional[base.Boolean] = True,
             parse_mode: typing.Optional[base.String] = None,
-            timeout: typing.Optional[typing.Union[base.Integer, base.Float, aiohttp.ClientTimeout]] = None
+            timeout: typing.Optional[typing.Union[base.Integer, base.Float, aiohttp.ClientTimeout]] = None,
+            server: TelegramAPIServer = TELEGRAM_PRODUCTION
     ):
         """
         Instructions how to get Bot token is found here: https://core.telegram.org/bots#3-how-do-i-create-a-bot
@@ -53,52 +57,71 @@ class BaseBot:
         :type parse_mode: :obj:`str`
         :param timeout: Request timeout
         :type timeout: :obj:`typing.Optional[typing.Union[base.Integer, base.Float, aiohttp.ClientTimeout]]`
+        :param server: Telegram Bot API Server endpoint.
+        :type server: :obj:`TelegramAPIServer`
         :raise: when token is invalid throw an :obj:`aiogram.utils.exceptions.ValidationError`
         """
+        self._main_loop = loop
+
         # Authentication
         if validate_token:
             api.check_token(token)
         self._token = None
         self.__token = token
+        self.id = int(token.split(sep=':')[0])
+        self.server = server
 
         self.proxy = proxy
         self.proxy_auth = proxy_auth
 
-        # Asyncio loop instance
-        if loop is None:
-            loop = asyncio.get_event_loop()
-        self.loop = loop
-
         # aiohttp main session
         ssl_context = ssl.create_default_context(cafile=certifi.where())
 
+        self._session: Optional[aiohttp.ClientSession] = None
+        self._connector_class: Type[aiohttp.TCPConnector] = aiohttp.TCPConnector
+        self._connector_init = dict(limit=connections_limit, ssl=ssl_context)
+
         if isinstance(proxy, str) and (proxy.startswith('socks5://') or proxy.startswith('socks4://')):
             from aiohttp_socks import SocksConnector
-            from aiohttp_socks.helpers import parse_socks_url
+            from aiohttp_socks.utils import parse_proxy_url
 
-            socks_ver, host, port, username, password = parse_socks_url(proxy)
+            socks_ver, host, port, username, password = parse_proxy_url(proxy)
             if proxy_auth:
                 if not username:
                     username = proxy_auth.login
                 if not password:
                     password = proxy_auth.password
 
-            connector = SocksConnector(socks_ver=socks_ver, host=host, port=port,
-                                       username=username, password=password,
-                                       limit=connections_limit, ssl_context=ssl_context,
-                                       rdns=True, loop=self.loop)
-
+            self._connector_class = SocksConnector
+            self._connector_init.update(
+                socks_ver=socks_ver, host=host, port=port,
+                username=username, password=password, rdns=True,
+            )
             self.proxy = None
             self.proxy_auth = None
-        else:
-            connector = aiohttp.TCPConnector(limit=connections_limit, ssl=ssl_context, loop=self.loop)
+
         self._timeout = None
         self.timeout = timeout
 
-        self.session = aiohttp.ClientSession(connector=connector, loop=self.loop, json_serialize=json.dumps)
-
         self.parse_mode = parse_mode
 
+    def get_new_session(self) -> aiohttp.ClientSession:
+        return aiohttp.ClientSession(
+            connector=self._connector_class(**self._connector_init, loop=self._main_loop),
+            loop=self._main_loop,
+            json_serialize=json.dumps
+        )
+
+    @property
+    def loop(self) -> Optional[asyncio.AbstractEventLoop]:
+        return self._main_loop
+
+    @property
+    def session(self) -> Optional[aiohttp.ClientSession]:
+        if self._session is None or self._session.closed:
+            self._session = self.get_new_session()
+        return self._session
+
     @staticmethod
     def _prepare_timeout(
             value: typing.Optional[typing.Union[base.Integer, base.Float, aiohttp.ClientTimeout]]
@@ -156,6 +179,8 @@ class BaseBot:
         finally:
             self._ctx_token.reset(token)
 
+    @deprecated("This method's behavior will be changed in aiogram v3.0. "
+                "More info: https://core.telegram.org/bots/api#close", stacklevel=3)
     async def close(self):
         """
         Close all client sessions
@@ -180,7 +205,7 @@ class BaseBot:
         :rtype: Union[List, Dict]
         :raise: :obj:`aiogram.exceptions.TelegramApiError`
         """
-        return await api.make_request(self.session, self.__token, method, data, files,
+        return await api.make_request(self.session, self.server, self.__token, method, data, files,
                                       proxy=self.proxy, proxy_auth=self.proxy_auth, timeout=self.timeout, **kwargs)
 
     async def download_file(self, file_path: base.String,
@@ -220,7 +245,7 @@ class BaseBot:
         return dest
 
     def get_file_url(self, file_path):
-        return api.Methods.file_url(token=self.__token, path=file_path)
+        return self.server.file_url(token=self.__token, path=file_path)
 
     async def send_file(self, file_type, method, file, payload) -> Union[Dict, base.Boolean]:
         """
@@ -260,10 +285,14 @@ class BaseBot:
             if value not in ParseMode.all():
                 raise ValueError(f"Parse mode must be one of {ParseMode.all()}")
             setattr(self, '_parse_mode', value)
+            if value == 'markdown':
+                warnings.warn("Parse mode `Markdown` is legacy since Telegram Bot API 4.5, "
+                              "retained for backward compatibility. Use `MarkdownV2` instead.\n"
+                              "https://core.telegram.org/bots/api#markdown-style", stacklevel=3)
 
     @parse_mode.deleter
     def parse_mode(self):
         self.parse_mode = None
 
     def check_auth_widget(self, data):
-        return check_token(data, self.__token)
+        return check_integrity(self.__token, data)
